package cn.ninegame.genericframework.module;

import android.content.Context;

import android.util.Log;

import org.osgi.framework.BundleException;

import java.io.File;
import java.net.URL;
import java.util.Dictionary;

import cn.ninegame.genericframework.basic.ExceptionManager;
import cn.ninegame.genericframework.basic.FrameworkFacade;
import cn.ninegame.genericframework.basic.FrameworkService;
import cn.ninegame.genericframework.basic.IController;
import cn.ninegame.genericframework.config.ModulePathManager;
import cn.ninegame.genericframework.tools.LOG;
import cn.ninegame.genericframework.tools.ReflectHelper;
import cn.ninegame.genericframework.ui.BaseDialogFragment;
import cn.ninegame.genericframework.ui.BaseFragment;

/**
 * Created by lingcong on 2015/6/2.
 */
public class ModuleEntryDynamic extends AbsModuleEntry {
    private static final String TAG = "ModuleEntryDynamic";

    private Context mContext;
    @Override
    public IController loadController(String controllerID) {
//        if (!loadModuleIfNeed()) {
//            Log.e(TAG, "Failed to load " + controllerID);
//            return null;
//        }
        IController controller = (IController) ReflectHelper.loadClass(controllerID);
        if(controller != null){
            controller.setModuleEntry(this);
        }
        return  controller;
    }

    @Override
    public Object loadControllerObject(String controllerID) {
        Object controllerObject = ReflectHelper.loadClass(controllerID);
        if(controllerObject != null){
            IController controller = (IController) controllerObject;
            controller.setModuleEntry(this);
        }
        return  controllerObject;
    }


    @Override
    public BaseFragment loadFragment(String fragmentName) {
//        if (!loadModuleIfNeed()) {
//            Log.e(TAG, "Failed to load " + fragmentName);
//            return null;
//        }

        BaseFragment fragment = (BaseFragment) ReflectHelper.loadClass(fragmentName);
        if (fragment != null) {
            fragment.setModuleEntry(this);
        }
        return fragment;
    }

    @Override
    public BaseDialogFragment loadDialogFragment(String fragmentName) {
//        if (!loadModuleIfNeed()) {
//            Log.e(TAG, "Failed to load " + fragmentName);
//            return null;
//        }

        BaseDialogFragment dialogFragment = (BaseDialogFragment) ReflectHelper.loadClass(fragmentName);
        if (dialogFragment != null) {
            dialogFragment.setModuleEntry(this);
        }
        return dialogFragment;
    }



    protected synchronized boolean loadModule() {
        boolean result = true;
        try {
            notifyOnLoadStart();

            mContext = doLoadModule();

            if (mApplicationID != null && mApplication == null) {
                mApplication = (ModuleApplication) ReflectHelper.loadClass(mApplicationID);
                if (mApplication != null) {
                    mApplication.setModuleEntry(this);
                    mApplication.setEnvironment(FrameworkFacade.getInstance().getEnvironment());
                    mApplication.onCreate();
                }
            }

            notifyOnLoadFinish();
        } catch (Exception e) {
            Log.e(TAG, "", e);

            notifyOnLoadError();

            FrameworkService.getInstance().getService(ExceptionManager.class).handleException(e);
            result = false;
        }

        return result;
    }

    private Context doLoadModule() throws Exception {
        ModuleInfo info = getModuleInfo();
        if(info.isAndroidLibrary){
            mLoadState=LOAD_STATE_LOADED;
            return mBaseContext.getApplicationContext();
        }
        // do load dex from apk
        ModuleLoader moduleLoader = new ModuleLoader();
        moduleLoader.load(mBaseContext, info);

        // load resources and create module context from apk.
        return ModuleContextLoader.loadContext(mBaseContext, info, moduleLoader.getModuleApkDirType(info.id, info.version));
    }

    public boolean isNotLoad(){
        return mLoadState == LOAD_STATE_NOT_LOAD;
    }

    public boolean isNeverLoad(){
        return isNeedOdex();
    }

    public boolean isNeedOdex(){
        File odexFile = ModulePathManager.getInstance().getModuleOdex(mModuleInfo.getID(),mModuleInfo.getVersion());
        if(odexFile != null && !odexFile.exists()){
            return true;
        }
        return false;
    }

    @Override
    public Context getContext() {
        return mContext;
    }

    @Override
    public int getModuleType() {
        return ModuleType.DYNAMIC;
    }

}
