package cn.ninegame.genericframework.module;

import java.io.File;
import java.util.List;
import java.util.concurrent.ConcurrentLinkedQueue;

import cn.ninegame.genericframework.config.DynamicLoadConfig;
import cn.ninegame.genericframework.tools.DeviceInfoUtil;
import cn.ninegame.genericframework.tools.FileUtil;
import cn.ninegame.genericframework.tools.LOG;
import cn.ninegame.genericframework.tools.dynamicload.DexLoader;

/**
 * <p>Title: ucweb</p>
 * <p/>
 * <p>Description: </p>
 *
 *  作用：闲时生成 module 的 odex 文件
 *
 *  背景：Module 加载时跳过了生成odex的步骤，没有odex文件生成
 *
 * <p>Copyright: Copyright (c) 2016</p>
 * <p/>
 * <p>Company: ucweb.com</p>
 *
 * @author ZhuZhiFan (zhifan.zzf@alibaba-inc.com)
 * @date 2016/6/21
 */
public class ModuleOdexMaker {

    private volatile static ModuleOdexMaker mInstance;
    private static final String TAG = "ModuleOdexMaker";
    // 模块Odex加载的时间间隔
    private static final long DEFAULT_INTERVAL = 3000;
    // 判断 CPU 是否处于忙时的阈值
    private float CPU_BUSY_THRESHOLD = 0.6f;

    private boolean mIsStart;
    private Thread mThread;
    private ConcurrentLinkedQueue<Object> mList;

    private ModuleOdexMaker() {
        mList = new ConcurrentLinkedQueue<Object>();
    }

    public static ModuleOdexMaker getInstance() {
        if (mInstance == null) {
            synchronized (ModuleOdexMaker.class) {
                if (mInstance == null) {
                    mInstance = new ModuleOdexMaker();
                }
            }
        }
        return mInstance;
    }


    public void makeOdex(ClassLoader loader, File dexDir, List<File> files, File odexFile) {
        LOG.d(TAG, "makeOdex: add task,  " + files.get(0).getName());
        mList.offer(new Object[] {loader, dexDir, files, odexFile});

        if (!mIsStart) {
            mIsStart = true;
            LOG.d(TAG, "makeOdex: new Thread");
            mThread = new Thread(new Runnable() {
                @Override
                public void run() {
                    while (!mList.isEmpty()) {
                        Object[] args = (Object[]) mList.peek();
                        File odexFile = (File) args[3];
                        File dir = (File) args[1];
                        String finishPath = dir.getAbsolutePath() + File.separator + DynamicLoadConfig.MODULE_ODEX_FINISH;
                        if (odexFile.exists() && new File(finishPath).exists()) { // odex  文件存在
                            mList.remove(args);
                        } else {
                            if (DeviceInfoUtil.getProcessCpuUsage() <= CPU_BUSY_THRESHOLD) {
                                mList.remove(args);
                                try {
                                    LOG.d(TAG, "ModuleLoader makeOdex: " + ((File) ((List) args[2]).get(0)).getName());
                                    DexLoader.installDexes((ClassLoader) args[0], dir, (List) args[2]);
                                    FileUtil.createNewFile(finishPath);
                                } catch (Exception e) {
                                    e.printStackTrace();
                                }
                            }

                            try {
                                Thread.sleep(DEFAULT_INTERVAL);
                            } catch (InterruptedException e) {
                                e.printStackTrace();
                            }

                        }
                    }

                    mIsStart = false;
                }
            });
            mThread.start();
        }
    }
}
