package cn.ninegame.genericframework.tools;

import java.io.FileOutputStream;
import java.nio.channels.FileLock;

/**
 * <p>Title: ucweb</p>
 * <p/>
 * <p>Description: </p>
 * 进程间文件锁
 * <p>Copyright: Copyright (c) 2016</p>
 * <p/>
 * <p>Company: ucweb.com</p>
 *
 * @author ZhuZhiFan (zhifan.zzf@alibaba-inc.com)
 * @date 2016/4/5
 */
public class FileLocker {
    private FileOutputStream mFileOutputStream;
    private FileLock mLock;

    /**
     * @param path 文件锁路径
     */
    public FileLocker(String path){
        try {
            mFileOutputStream = new FileOutputStream(path);
        } catch (Throwable t) {
            if (mFileOutputStream != null) {
                try {
                    mFileOutputStream.close();
                } catch (Throwable tt) {
                    tt.printStackTrace();
                } finally {
                    mFileOutputStream = null;
                }

            }
        }
    }

    /**
     * 获取进程锁
     * @param block 如果文件已被锁定，是否阻塞等待
     * @return 锁定成功则返回true
     */
    public synchronized  boolean lock(boolean block){
        if (mFileOutputStream == null){
            return false;
        }

        try {
            if (block) {
                mLock = mFileOutputStream.getChannel().lock();
            } else {
                mLock = mFileOutputStream.getChannel().tryLock();
            }
        } catch (Throwable t) {
            if (mLock != null) {
                try {
                    mLock.release();
                } catch (Throwable t1) {
                    t1.printStackTrace();
                } finally {
                    mLock = null;
                }
            }
            t.printStackTrace();
        }

        return mLock != null;
    }

    /**
     * 释放进程文件锁
     */
    public synchronized void release() {
        if (mFileOutputStream == null) {
            return;
        }

        unlock();

        try {
            mFileOutputStream.close();
        } catch (Throwable t) {
            t.printStackTrace();
        } finally {
            mFileOutputStream = null;
        }
    }

    private  synchronized void unlock() {
        if (mLock == null) {
            return;
        }

        try {
            mLock.release();
        } catch (Throwable t) {
            t.printStackTrace();
        }finally {
            mLock = null;
        }
    }
}
