package cn.ninegame.genericframework.tools;

/**
 * Created by Administrator on 2017-07-31.
 */

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class GundamFileLock {

    private static GundamFileLock singleton;

    public static GundamFileLock getInstance(){
        if (singleton == null){
            singleton = new GundamFileLock();
        }
        return singleton;
    }

    private class FileLockCount{
        RandomAccessFile fOs;
        FileChannel fChannel;
        FileLock mFileLock;
        int mRefCount;

        FileLockCount(FileLock mFileLock, int mRefCount, RandomAccessFile fOs, FileChannel fChannel){
            this.mFileLock = mFileLock;
            this.mRefCount = mRefCount;
            this.fOs = fOs;
            this.fChannel = fChannel;
        }
    }

    /*
     * The FileLock is interesting, 1st not suitable for intraprocess
     * i.e., once the current process already hold the lock, another
     * thread's aquirement would be always success.
     * 2nd is that FileLock has no reference count, i.e., Once both
     * threads hold the lock, any thread release that would totally
     * release the FileLock.
     * To resolve the 1st issue, we used BundleLock.
     * To resolve the 2nd issue, we need add ref count ourselves.
     */
    private Map<String, FileLockCount> mRefCountMap = new ConcurrentHashMap<String, FileLockCount>();

    /*
     * Refcnt increment
     * return the ref count of current file's filelock
     */
    private int RefCntInc(String name, FileLock fileLock, RandomAccessFile fOs, FileChannel fChannel){
        Integer val = 0;

        if (mRefCountMap.containsKey(name)){
            val = mRefCountMap.get(name).mRefCount++;
        } else {
            val = 1;
            FileLockCount newFileLockCount = new FileLockCount(fileLock, val, fOs, fChannel);
            mRefCountMap.put(name, newFileLockCount);
        }

        return val;
    }

    /*
     * Refcnt decrement
     * return the ref count of current file's filelock
     */
    private int RefCntDec(String name){
        Integer val = 0;

        if (mRefCountMap.containsKey(name)){
            val = --mRefCountMap.get(name).mRefCount;
         /*
          * Remove the item once value is already 0
          */
            if (val <= 0){
                mRefCountMap.remove(name);
            }
        }

        return val;
    }

    /*
     * Hold a exclusive lock, file is the directory
     */
    public boolean LockExclusive(File orgFile) {
        RandomAccessFile fOs = null;
        FileChannel fChannel = null;
        try {
            if (orgFile == null){
                return false;
            }

            File file ;
            if(orgFile.exists() && orgFile.isDirectory()){
                file = new File(orgFile.getAbsolutePath().concat("/lock"));
            }else{
                file = new File(orgFile.getParentFile().getAbsolutePath().concat("/lock"));
            }
            if (file.exists() != true){
                file.createNewFile();
            }

            fOs = new RandomAccessFile(file.getAbsolutePath(), "rw");
            fChannel = fOs.getChannel();

            FileLock fFileLock = fChannel.lock();
            if(fFileLock.isValid()) {
                RefCntInc(file.getAbsolutePath(), fFileLock, fOs, fChannel);
                return true;
            }
        }catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }


    /*
     * Free file lock, do remember to invoke it once caught exception
     */
    public void unLock(File orgFile) {

        File file ;
        if(orgFile.exists() && orgFile.isDirectory()){
            file = new File(orgFile.getAbsolutePath().concat("/lock"));
        }else{
            file = new File(orgFile.getParentFile().getAbsolutePath().concat("/lock"));
        }

        if (file.exists() == false){
            return;
        }

        if ((file != null) && !mRefCountMap.containsKey(file.getAbsolutePath())){
            return;
        }

        FileLockCount flc = mRefCountMap.get(file.getAbsolutePath());
        if (flc == null){
            return;
        }

        FileLock fl = flc.mFileLock;
        RandomAccessFile fOs = flc.fOs;
        FileChannel fChannel = flc.fChannel;

        try {
            if (RefCntDec(file.getAbsolutePath()) <= 0){
                if (fl != null && fl.isValid()){
                    fl.release();
                }
                if (fOs != null){
                    fOs.close();
                }
                if (fChannel != null){
                    fChannel.close();
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


}